"""Test Run Module in Unit Test Package

@(#) testRun.py 1.3@(#)"""

import sys
import time


class TestRun:
    """Coordinator for running test suites."""

    def __init__ (self, name, resultStream = None, statStream = None):
	"""Initialize with ..."""
	self.name = name
	self.resultStream = resultStream
	self.statStream = statStream
	self.testSuites = []

    def addSuite (self, aSuite):
	"""Add a test suite."""
	self.testSuites.append (aSuite)

    def addSuites (self, suites):
	"""Add a list of test suites."""
	self.testSuites = self.testSuites + suites

    def setUp (self):
	"""Set up a fixture for a test."""
	pass

    def tearDown (self):
	"""Tear down the test fixture."""
	pass

    def start (self):
	"""Note the start time."""
	self.startTime = time.time ()

    def stop (self):
	"""Note the stop time."""
	self.stopTime = time.time ()

    def runSuites (self):
	"""Run all test suites."""
	self.results = []
	for aTestSuite in self.testSuites:
	    self.results.append (aTestSuite.run ())

    def runTest (self):
	"""Run test suites in timed, controlled environment."""
	self.start ()
	self.setUp ()
	self.runSuites ()
	self.tearDown ()
	self.stop ()

    def printHeadersOn (self, aStream):
	"""Print the test result header on a stream."""
	aStream.write ("Summary test results for %s\n" % self.name)
	aStream.write ("    Start time: %s\n" % time.ctime (self.startTime))
	aStream.write ("    Stop time: %s\n\n" % time.ctime (self.stopTime))

    def printSummaryOn (self, aStream):
	"""Print summary statistics about results."""
	aStream.write ("    Test suites: %d\n" % self.suiteCount ())
	aStream.write ("    Suites with failures: %d\n" % self.failureCount ())
	aStream.write ("    Suites with errors: %d\n\n" % self.errorCount ())

    def printResultsOn (self, aStream):
	"""Print actual results."""
	for result in self.results:
	    result.printOn (aStream)

    def _printOn (self, aStream):
	"""Print everything on a stream."""
	self.printHeadersOn (aStream)
	self.printSummaryOn (aStream)
	self.printResultsOn (aStream)

    def printOn (self, aStream = None):
	"""Print everything to appropriate stream."""
	self._printOn (aStream or self.resultStream or sys.stdout)

    def formatTimeForStatistics (self, dateTime):
	"""Format a time for summary statistics."""
	return time.strftime ("%Y" "%m" "%d\t%H:%M:%S", time.localtime (dateTime))

    def printSummaryStatisticsOn (self, aStream):
	"""Print summary statistics on a stream."""
	aStream.write ("%s\t%s\t%s\t%d\t%d\t%d\n" %
		(self.name,
		self.formatTimeForStatistics (self.startTime),
		self.formatTimeForStatistics (self.stopTime),
		self.suiteCount (),
		self.failureCount (),
		self.errorCount ()))

    def printResultStatisticsOn (self, aStream):
	"""Print statistics about results on a stream."""
	for result in self.results:
	    result.printSummaryOn (aStream)

    def _printStatisticsOn (self, aStream):
	"""Print statistics on a stream."""
	self.printSummaryStatisticsOn (aStream)
	self.printResultStatisticsOn (aStream)

    def printStatisticsOn (self, aStream = None):
	"""Print statistics to appropriate stream."""
	self._printStatisticsOn (aStream or self.statStream or sys.stdout)

    def suiteCount (self):
	"""Return the number of test suites."""
	return len (self.testSuites)

    def failureCount (self):
	"""Return the number of failed test suites."""
	return reduce (
		lambda count, result: count + (not result.hasNoFailures ()),
		self.results, 0)

    def errorCount (self):
	"""Return the number of errored test suites."""
	return reduce (
		lambda count, result: count + (not result.hasNoErrors ()),
		self.results, 0)

    def run (self, resultStream = None, statStream = None):
	"""Run some test suites and report the results."""
	self.runTest ()
	self.printOn (resultStream)
	self.printStatisticsOn (statStream)
