"""Test Suite Module of Unit Test Package

@(#) testSuite.py 1.5@(#)"""

import testResult
from types import FunctionType

class TestSuite:
    """A suite of tests."""

    def __init__ (self, name, testCaseClass = None, initialTestCases = None):
	"""Initialize with a name and initial test cases."""
	self.name = name
	self.testCases = []
	if testCaseClass:
	    self.addTestCasesFromClass (testCaseClass)
	if initialTestCases:
	    self.addTestCases (initialTestCases)

    def getName (self):
	"""Return the name."""
	return self.name

    def getTestCaseCount (self):
	"""Return the count of test cases."""
	return len (self.testCases)

    def addTestCase (self, aTestCase):
	"""Add a test case."""
	self.testCases.append (aTestCase)

    def addTestCases (self, testCases):
	"""Add a list of test cases."""
	self.testCases = self.testCases + testCases

    def addTestCasesFromClass (self, testCaseClass):
	"""Add test cases for all test case class methods that start with "test"."""
	for name, value in testCaseClass.__dict__.items ():
	    if name [0:4] == "test" and type (value) == FunctionType:
		self.addTestCase (testCaseClass (name))

    def defaultTestResultClass (self):
	"""Return the default test result class."""
	return testResult.TestResult

    def defaultTestResult (self):
	"""Return a default test result."""
	return self.defaultTestResultClass () (self)

    def run (self):
	"""Run a suite, returning the results."""
	result = self.defaultTestResult ()
	result.start ()
	self.runTestResult (result)
	result.stop ()
	return result

    def runTestResult (self, aTestResult):
	"""Run a suite, saving the results."""
	for aTestCase in self.testCases:
	    aTestCase.run (aTestResult)

