from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem            
import Acquisition               
import AccessControl.Role      

import string

from WidgetFinder import WidgetFinder

class ZWidgetManager(
    OFS.SimpleItem.Item,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager
    ):

    meta_type = "XMLWidgetManager"

    # the various tabs
    manage_options = (
        {'label' : 'Edit',     'action' : 'manage_main'},
        {'label' : 'Security', 'action' : 'manage_access'},
        )

    # display edit screen as main management screen
    manage_main = HTMLFile('widgetmanagerEdit', globals())

    # permissions
    __ac_permissions__ = (
        ('View Management Screens',
         ('manage_tabs',
          'manage_main',
          )
         ),

        ('Change permissions',
         ('manage_access',)),
        
        ('Access Contents Information',        
         ('__call__',
          )
         ),
        )

    def __init__(self, id, title):
        """Create a new widget manager.
        """
        self.id = id
        self.title = title
        self.widgetnames = []
        self.widgetfinder = WidgetFinder()
                
    def manage_edit(self, title, REQUEST=None):
        """Handle result from edit action.
        title - the new title of the manager
        REQUEST - contains other data
        """
        # set title
        self.title = title
                
        if REQUEST is not None:
            # extract list of widget names
            widgetnames = map(string.strip, string.split(REQUEST['widgets'], "\n"))
            # filter out all empty names
            widgetnames = filter(lambda name: name != "", widgetnames)
            
            # add widgets
            self.addWidgets(widgetnames)
            
        # done with editing
        if REQUEST is not None:
            message = "Properties changed"
            return self.manage_main(self, REQUEST,
                                    manage_tabs_message=message)

    def addWidgets(self, widgetnames):
        """Add a number of widgets to the widgetfinder.
        """
        # store the names in this object
        self.widgetnames = widgetnames

        # now add them to the widget finder
        widgetfinder = self.widgetfinder
        
        for widgetname in widgetnames:
            widget = getattr(self, widgetname)
            widgetfinder.addWidget(widget)

        self.widgetfinder = widgetfinder
        
    def clearWidgets(self):
        """Clear all widgets.
        """
        # remove widgetnames from this object
        self.widgetnames = []
        # clear all widgets from widgetfinder
        self.widgetfinder.clearLevel()
        self.widgetfinder = self.widgetfinder

    def test_find(self, REQUEST):
        """Find the widget.
        """
        widget = self.widgetfinder.find(REQUEST.current_node)
        if widget:
            return "found a widget"
        else:
            return "didn't find a widget"

    index_html = None

    def __call__(me, self, REQUEST):
        """Find the widget to draw with.
        """
        node = self
        widget = me.widgetfinder.find(node)
        # if we found the widget
        if widget:
            return getattr(node, widget.id)
        else:
            return "didn't find it"
             
# form to add the WidgetManager to a folder
manage_addWidgetManagerForm = HTMLFile('widgetmanagerAdd', globals())

# adds WidgetManager to folder
def manage_addWidgetManager(self, id, title='', REQUEST=None):
    """Add manager to folder.
    """
    wm = ZWidgetManager(id, title)
    
    self._setObject(id, wm)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

    
